@@ -1,30 +1,7 @@
 //! Encoded pointer to a span region.
 
-use crate::{
-	domain::Domain,
-	index::{
-		BitIdx,
-		BitTail,
-	},
-	mem::BitMemory,
-	mutability::{
-		Const,
-		Mut,
-		Mutability,
-	},
-	order::{
-		BitOrder,
-		Lsb0,
-	},
-	ptr::{
-		Address,
-		BitPtr,
-		BitPtrError,
-	},
-	slice::BitSlice,
-	store::BitStore,
-};
-
+#[cfg(any(feature = "alloc", test))]
+use core::convert::TryInto;
 use core::{
 	any,
 	convert::Infallible,
@@ -42,10 +19,32 @@ use core::{
 	},
 };
 
+use funty::IsNumber;
 use wyz::fmt::FmtForward;
 
-#[cfg(any(feature = "alloc", test))]
-use core::convert::TryInto;
+use super::{
+	Address,
+	BitPtr,
+	BitPtrError,
+	BitPtrRange,
+	Const,
+	Mut,
+	Mutability,
+};
+use crate::{
+	domain::Domain,
+	index::{
+		BitIdx,
+		BitTail,
+	},
+	mem::BitMemory,
+	order::{
+		BitOrder,
+		Lsb0,
+	},
+	slice::BitSlice,
+	store::BitStore,
+};
 
 /** Encoded handle to a bit-precision memory region.
 
@@ -277,17 +276,19 @@ where
 	/// received from the allocation system are required to satisfy this
 	/// constraint, so a failure is an exceptional program fault rather than an
 	/// expected logical mistake.
+	#[inline(always)]
 	#[cfg(feature = "alloc")]
 	#[cfg(not(tarpaulin_include))]
 	pub(crate) fn uninhabited(addr: Address<M, T>) -> Self {
 		Self {
-			ptr: addr.to_nonnull().cast::<()>(),
+			ptr: addr.into_inner().cast::<()>(),
 			len: 0,
 			_or: PhantomData,
 			_ty: PhantomData,
 		}
 	}
 
+	#[inline]
 	pub(crate) fn new(
 		addr: Address<M, T>,
 		head: BitIdx<T::Mem>,
@@ -325,13 +326,14 @@ where
 	/// See [`::new`].
 	///
 	/// [`::new`]: Self::new
+	#[inline]
 	pub(crate) unsafe fn new_unchecked(
 		addr: Address<M, T>,
 		head: BitIdx<T::Mem>,
 		bits: usize,
 	) -> Self {
-		let head = head.value() as usize;
-		let ptr_data = addr.value() & Self::PTR_ADDR_MASK;
+		let head = head.into_inner() as usize;
+		let ptr_data = addr.to_const() as usize & Self::PTR_ADDR_MASK;
 		let ptr_head = head >> Self::LEN_HEAD_BITS;
 
 		let len_head = head & Self::LEN_HEAD_MASK;
@@ -381,6 +383,7 @@ where
 	///
 	/// [`::from_bitslice_ptr()`]: Self::from_bitslice_ptr
 	//  Mutable or immutable span descriptors can become immutable pointers.
+	#[inline]
 	pub(crate) fn to_bitslice_ptr(self) -> *const BitSlice<O, T> {
 		ptr::slice_from_raw_parts(
 			self.ptr.as_ptr() as *const u8 as *const (),
@@ -420,6 +423,7 @@ where
 	/// This does not alter the encoded value of the pointer! It only
 	/// reinterprets the element type, and the encoded value may shift
 	/// significantly in the result type. Use with caution.
+	#[cfg_attr(not(tarpaulin_include), inline(always))]
 	pub(crate) fn cast<U>(self) -> BitSpan<M, O, U>
 	where U: BitStore {
 		let Self { ptr, len, .. } = self;
@@ -496,9 +500,9 @@ where
 					runs for the remaining bits in it, and all the bits of `l`.
 					*/
 					Some((head, addr)) => BitSpan::new_unchecked(
-						Address::new_unchecked(addr as *const _ as usize),
+						Address::new(NonNull::from(addr)),
 						head,
-						t_bits - head.value() as usize + l_bits,
+						t_bits - head.into_inner() as usize + l_bits,
 					),
 					//  If the head does not exist, then the left span only
 					//  covers `l`. If `l` is empty, then so is the span.
@@ -508,7 +512,7 @@ where
 						}
 						else {
 							BitSpan::new_unchecked(
-								Address::new_unchecked(l_addr as usize),
+								Address::new(NonNull::new(l_addr).unwrap()),
 								BitIdx::ZERO,
 								l_bits,
 							)
@@ -521,7 +525,7 @@ where
 				}
 				else {
 					BitSpan::new_unchecked(
-						Address::new_unchecked(c_addr as usize),
+						Address::new(NonNull::new(c_addr).unwrap()),
 						BitIdx::ZERO,
 						c_bits,
 					)
@@ -543,13 +547,13 @@ where
 						//  If the `r` slice exists, then the right span
 						//  *begins* in it.
 						if r.is_empty() {
-							Address::new_unchecked(addr as *const T as usize)
+							Address::new(NonNull::from(addr))
 						}
 						else {
-							Address::new_unchecked(r_addr as *const T as usize)
+							Address::new(NonNull::new(r_addr).unwrap())
 						},
 						BitIdx::ZERO,
-						tail.value() as usize + r_bits,
+						tail.into_inner() as usize + r_bits,
 					),
 					//  If the tail does not exist, then the right span is only
 					//  `r`.
@@ -557,7 +561,7 @@ where
 						//  If `r` exists, then the right span covers it.
 						if !r.is_empty() {
 							BitSpan::new_unchecked(
-								Address::new_unchecked(r_addr as usize),
+								Address::new(NonNull::new(r_addr).unwrap()),
 								BitIdx::ZERO,
 								r_bits,
 							)
@@ -587,11 +591,12 @@ where
 	/// The address of the starting element of the memory region. This address
 	/// is weakly typed so that it can be cast by call sites to the most useful
 	/// access type.
+	#[inline]
 	pub(crate) fn address(&self) -> Address<M, T> {
 		unsafe {
-			Address::new_unchecked(
-				self.ptr.as_ptr() as usize & Self::PTR_ADDR_MASK,
-			)
+			Address::new(NonNull::new_unchecked(
+				(self.ptr.as_ptr() as usize & Self::PTR_ADDR_MASK) as *mut T,
+			))
 		}
 	}
 
@@ -608,6 +613,7 @@ where
 	/// None. The invariants of [`::new`] must be checked at the caller.
 	///
 	/// [`::new`]: Self::new
+	#[inline]
 	#[cfg(any(feature = "alloc", test))]
 	pub(crate) unsafe fn set_address<A>(&mut self, addr: A)
 	where
@@ -615,7 +621,7 @@ where
 		A::Error: Debug,
 	{
 		let addr = addr.try_into().unwrap();
-		let mut addr_value = addr.value();
+		let mut addr_value = addr.to_const() as usize;
 		addr_value &= Self::PTR_ADDR_MASK;
 		addr_value |= self.ptr.as_ptr() as usize & Self::PTR_HEAD_MASK;
 		self.ptr = NonNull::new_unchecked(addr_value as *mut ());
@@ -634,6 +640,7 @@ where
 	///
 	/// [`BitIdx`]: crate::index::BitIdx
 	/// [`self.address()`]: Self::pointer
+	#[inline]
 	pub(crate) fn head(&self) -> BitIdx<T::Mem> {
 		//  Get the high part of the head counter out of the pointer.
 		let ptr = self.ptr.as_ptr() as usize;
@@ -655,9 +662,10 @@ where
 	///
 	/// `head` is written into the `.head` logical field, without affecting
 	/// `.addr` or `.bits`.
+	#[inline]
 	#[cfg(any(feature = "alloc", test))]
 	pub(crate) unsafe fn set_head(&mut self, head: BitIdx<T::Mem>) {
-		let head = head.value() as usize;
+		let head = head.into_inner() as usize;
 		let mut ptr = self.ptr.as_ptr() as usize;
 
 		ptr &= Self::PTR_ADDR_MASK;
@@ -677,6 +685,7 @@ where
 	/// # Returns
 	///
 	/// A count of how many live bits the region pointer describes.
+	#[cfg_attr(not(tarpaulin_include), inline(always))]
 	pub(crate) fn len(&self) -> usize {
 		self.len >> Self::LEN_HEAD_BITS
 	}
@@ -694,6 +703,7 @@ where
 	/// The `new_len` value is written directly into the `.bits` logical field.
 	///
 	/// [`REGION_MAX_BITS`]: Self::REGION_MAX_BITS
+	#[cfg_attr(not(tarpaulin_include), inline(always))]
 	pub(crate) unsafe fn set_len(&mut self, new_len: usize) {
 		debug_assert!(
 			new_len <= Self::REGION_MAX_BITS,
@@ -705,10 +715,16 @@ where
 	}
 
 	/// Gets a pointer to the starting bit of the span.
+	#[inline]
 	pub(crate) fn as_bitptr(self) -> BitPtr<M, O, T> {
 		BitPtr::new(self.address(), self.head())
 	}
 
+	#[inline]
+	pub(crate) fn as_bitptr_range(self) -> BitPtrRange<M, O, T> {
+		unsafe { self.as_bitptr().range(self.len()) }
+	}
+
 	/// Gets the three logical components of the pointer.
 	///
 	/// # Parameters
@@ -721,6 +737,7 @@ where
 	/// - `.1`: The index of the first live bit in the first element of the
 	///   region.
 	/// - `.2`: The number of live bits in the region.
+	#[inline]
 	pub(crate) fn raw_parts(&self) -> (Address<M, T>, BitIdx<T::Mem>, usize) {
 		(self.address(), self.head(), self.len())
 	}
@@ -740,9 +757,10 @@ where
 	/// live bits included in the referent region.
 	///
 	/// [`self.address()`]: Self::pointer
+	#[inline]
 	pub(crate) fn elements(&self) -> usize {
 		//  Find the distance of the last bit from the base address.
-		let total = self.len() + self.head().value() as usize;
+		let total = self.len() + self.head().into_inner() as usize;
 		//  The element count is always the bit count divided by the bit width,
 		let base = total >> T::Mem::INDX;
 		//  plus whether any fractional element exists after the division.
@@ -763,16 +781,17 @@ where
 	/// T::Mem::BITS`.
 	///
 	/// It will be zero only when `self` is empty.
+	#[inline]
 	pub(crate) fn tail(&self) -> BitTail<T::Mem> {
 		let (head, len) = (self.head(), self.len());
 
-		if head.value() == 0 && len == 0 {
+		if head.into_inner() == 0 && len == 0 {
 			return BitTail::ZERO;
 		}
 
 		//  Compute the in-element tail index as the head plus the length,
 		//  modulated by the element width.
-		let tail = (head.value() as usize + len) & T::Mem::MASK as usize;
+		let tail = (head.into_inner() as usize + len) & T::Mem::MASK as usize;
 		/* If the tail is zero, wrap it to `T::Mem::BITS` as the maximal. This
 		upshifts `1` (tail is zero) or `0` (tail is not), then sets the upshift
 		on the rest of the tail, producing something in the range
@@ -800,9 +819,10 @@ where
 	/// stepping.
 	///
 	/// [`T::Mem`]: crate::store::BitStore::Mem
+	#[inline]
 	pub(crate) unsafe fn incr_head(&mut self) {
 		//  Increment the cursor, permitting rollover to `T::Mem::BITS`.
-		let head = self.head().value() as usize + 1;
+		let head = self.head().into_inner() as usize + 1;
 
 		//  Write the low bits into the `.len` field, then discard them.
 		self.len &= !Self::LEN_HEAD_MASK;
@@ -900,6 +920,7 @@ where
 	///
 	/// [`mem::transmute`]: core::mem::transmute
 	//  Immutable pointers can only become immutable span descriptors.
+	#[inline]
 	pub(crate) fn from_bitslice_ptr(raw: *const BitSlice<O, T>) -> Self {
 		let slice_nn = match NonNull::new(raw as *const [()] as *mut [()]) {
 			Some(nn) => nn,
@@ -914,22 +935,6 @@ where
 			_ty: PhantomData,
 		}
 	}
-
-	/// Assert that an immutable span pointer is in fact mutable.
-	///
-	/// This can only be called from a context where a mutable span descriptor
-	/// was lowered to immutable and needs to be re-raised; it is Undefined
-	/// Behavior in the compiler to call it on a span descriptor that was never
-	/// mutable.
-	pub(crate) unsafe fn assert_mut(self) -> BitSpan<Mut, O, T> {
-		let Self { ptr, len, _or, .. } = self;
-		BitSpan {
-			ptr,
-			len,
-			_or,
-			_ty: PhantomData,
-		}
-	}
 }
 
 impl<O, T> BitSpan<Mut, O, T>
@@ -983,6 +988,7 @@ where
 	O: BitOrder,
 	T: BitStore,
 {
+	#[inline(always)]
 	fn clone(&self) -> Self {
 		*self
 	}
@@ -1004,14 +1010,15 @@ where
 	T1: BitStore,
 	T2: BitStore,
 {
+	#[inline]
 	fn eq(&self, other: &BitSpan<M2, O, T2>) -> bool {
 		let (addr_a, head_a, bits_a) = self.raw_parts();
 		let (addr_b, head_b, bits_b) = other.raw_parts();
 		//  Since ::BITS is an associated const, the compiler will automatically
 		//  replace the entire function with `false` when the types don’t match.
 		T1::Mem::BITS == T2::Mem::BITS
-			&& addr_a.value() == addr_b.value()
-			&& head_a.value() == head_b.value()
+			&& addr_a.to_const() as usize == addr_b.to_const() as usize
+			&& head_a.into_inner() == head_b.into_inner()
 			&& bits_a == bits_b
 	}
 }
@@ -1155,25 +1162,33 @@ impl<T> std::error::Error for BitSpanError<T> where T: BitStore
 
 #[cfg(test)]
 mod tests {
+	use core::{
+		convert::TryFrom,
+		mem,
+		ptr,
+	};
+
+	use tap::Pipe;
+
 	use super::*;
 	use crate::{
 		prelude::*,
-		ptr::AddressError,
-	};
-	use core::{
-		mem,
-		ptr,
+		ptr::{
+			check_alignment,
+			MisalignError,
+			NullPtrError,
+		},
 	};
 
 	#[test]
 	fn ctor() {
 		assert!(matches!(
-			Address::<Const, u8>::new(0),
-			Err(AddressError::Null)
+			Address::<Const, u8>::try_from(ptr::null()),
+			Err(NullPtrError),
 		));
 		assert!(matches!(
-			Address::<Const, u16>::new(3),
-			Err(AddressError::Misaligned(addr)) if addr as usize == 3
+			Address::<Const, u16>::try_from(3 as *const u16).unwrap().pipe(check_alignment),
+			Err(MisalignError { ptr }) if ptr as usize == 3,
 		));
 
 		//  Double check the null pointers, but they are in practice impossible
@@ -1190,7 +1205,7 @@ mod tests {
 		let mut addr = Address::from(&data);
 		let head = BitIdx::new(5).unwrap();
 		assert!(BitSpan::<_, Lsb0, _>::new(addr, head, !3).is_err());
-		addr = unsafe { Address::new_unchecked(!1) };
+		addr = Address::try_from(!1 as *const u16).unwrap();
 		assert!(BitSpan::<_, Lsb0, _>::new(addr, head, 50).is_err());
 	}
 
@@ -1266,26 +1281,4 @@ mod tests {
 			mem::size_of::<*const [usize]>()
 		);
 	}
-
-	#[test]
-	#[cfg(feature = "alloc")]
-	fn render() {
-		#[cfg(not(feature = "std"))]
-		use alloc::format;
-
-		assert_eq!(
-			format!("{}", Address::<Const, u8>::new(0).unwrap_err()),
-			"`bitvec` will not operate on the null pointer"
-		);
-		assert_eq!(
-			format!("{}", Address::<Const, u16>::new(0x13579).unwrap_err()),
-			"`bitvec` requires that the address 0x13579 clear its least 1 bits \
-			 to be aligned for type u16"
-		);
-		assert_eq!(
-			format!("{}", Address::<Const, u32>::new(0x13579).unwrap_err()),
-			"`bitvec` requires that the address 0x13579 clear its least 2 bits \
-			 to be aligned for type u32"
-		);
-	}
 }
